# phy_access.py
#
# This file is part of python wrapper for the BROADWAY2 package.
#
# MIT License

# Copyright (c) 2013-2020 FibreCode GmbH <info@fibrecode.com>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


# generic SMI-Access via FC401 USB-Tool Adapter

# in difference to FC6xx USB-Adapters, Phy is external and unknown. Enables generic access with given PhyAddr

from broadway2 import broadway_api

class PhyAccess:
    def __init__(self, dev, phyAddr=-1, supported=0):
        """Default constructor
        dev is contrtol_device to USB-Adapter
        phyAddr is hw-address at SMI-Bus
        supported is 2 bits (CL45, CL22); 3=both,2=CL45,1=CL22,0=None
        """
        self._c45 = (supported & 2) == 2
        self._c22 = (supported & 1) == 1
        self.__device = dev
        self.__phyAddr = phyAddr

        if self._c45:
            if phyAddr >= 0:
                self.read_dev_reg = self.__read_reg45_ext
                self.write_dev_reg = self.__write_reg45_ext

        if self._c22:
            if phyAddr < 0:
                self.read_reg = self.__read_reg_int
                self.write_reg = self.__write_reg_int
            else:
                self.read_reg = self.__read_reg_ext
                self.write_reg = self.__write_reg_ext

    def get_device(self):
        """helper function to return current used device"""
        return self.__device
        
    def connect(self, dev):
        """connect to given device instance"""
        self.__device = dev

    def disconnect(self):
        """disconnect this instance from current USB HID device"""
        self.__device = None
    @property
    def supportsClause22(self):
        return self._c22

    @property
    def supportsClause45(self):
        return self._c45


    @property
    def isConnected(self):
        """return True/False if phy instance is connected to USB-Adapter"""
        return self.__device != None

    # external Phy connected to FC401 or internal using phyAddr=0
    def __read_reg_ext(self, regAddr):
        """read register access via MDIO on given phyAddr/register"""
        res = broadway_api.read_phy_register_ex(self.__device, self.__phyAddr, regAddr)
        return res

    def __write_reg_ext(self, regAddr, regValue):
        """write register access via MDIO on given phyAddr"""
        res = broadway_api.write_phy_register_ex(self.__device, self.__phyAddr, regAddr, regValue)
        return res

    # internal connected Phy direct on FC6xx on clause 22
    def __read_reg_int(self, register):
        res = broadway_api.read_phy_register(self.__device, register)
        return res

    def __write_reg_int(self, register, regValue):
        res = broadway_api.write_phy_register(self.__device, register, regValue)
        return res

    # connected Phy using clause 45
    def __read_reg45_ext(self, devAddr, regAddr):
        res = broadway_api.read_phy_register45_ex(self.__device, self.__phyAddr, devAddr, regAddr)
        return res

    def __write_reg45_ext (self, devAddr, regAddr, regValue):
        res = broadway_api.write_phy_register45_ex(self.__device, self.__phyAddr, devAddr, regAddr, regValue)
        return res
