# reg_decoder.py
#
# This file is part of python wrapper for the BROADWAY2 package.
#
# MIT License

# Copyright (c) 2013-2022 FibreCode GmbH <info@fibrecode.com>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import datetime
import json

class RegDecoder:
    def __init__(self, regMap):
        self.RegMap = regMap    # initialized with complete Register-Map generated by SysRapid. e.g. TJA1103A, Register-Dic, RegisterBits-Dic, ...

    def get_register(self):
        return self.RegMap.Register

    def get_register_sorted(self):
        res = []
        for reg in self.RegMap.Register:
            res.append((reg, self.RegMap.Register[reg]))
            
        return sorted(res)


    def get_register_details(self, regIndex):
        try:
            regBits = self.RegMap.RegisterBits[regIndex]
        except:
            regBits = None
             
        return regBits

    def lookup_register(self, regIndex):
        """lookup register offset and return tag"""
        try:
            tag = self.RegMap.Register[regIndex]['Tag']
        except:
            tag = 'N/A'
        return tag

    def lookup_reg_symbol(self, regIndex, symbol):
        """dynamic lookup to save huge dic"""
        regBits = self.RegMap.RegisterBits[regIndex]
        idx = 0
        for offset in regBits:
            if symbol == regBits[offset]['Symbol']:
                return (idx, offset, regBits[offset]['Width'])
            idx += 1
        return None


    def decode(self, reg, value):
        """generic bit decoder, based on register/Bit-Definitions"""
        res = []
        try:
            regDef = self.RegMap.RegisterBits[reg]
        except:
            return res

        for bits in regDef:
            try:
                bitDef = regDef[bits]
                mask = ((1 << (bits + bitDef['Width'])) - 1) & ~((1 << bits)-1)
            except:
                return res
            bitsValue = (value & mask) >> bits
            try:               
                if bitDef['Width'] == 1:
                    if bitsValue:
                        res.append((bits, bitDef['Width'], bitDef['Symbol'], bitDef['Values'][1], bitsValue))
                    else:
                        res.append((bits, bitDef['Width'],bitDef['Symbol'], bitDef['Values'][0], bitsValue))
                else:
                    if len(bitDef['Values']) > 0:
                        res.append((bits, bitDef['Width'], bitDef['Symbol'], bitDef['Values'][bitsValue], bitsValue))  # enum
                    else:
                        res.append((bits, bitDef['Width'], bitDef['Symbol'], '', bitsValue))   # just use integer value
            except:
                res.append((bits, 0, '', '', -1))
        return sorted(res, reverse=True)

    def decode_symbol(self, reg, symbol, value):
        """bit decoder for only one known symbol"""
        try:
            regDef = self.RegMap.RegisterBits[reg]
        except:
            return None

        for bits in regDef:
            if regDef[bits]['Symbol'] == symbol:
                try:
                    bitDef = regDef[bits]
                    mask = ((1 << (bits + bitDef['Width'])) - 1) & ~((1 << bits)-1)
                except:
                    return None
                bitsValue = (value & mask) >> bits
                try:               
                    if bitDef['Width'] == 1:
                        if bitsValue:
                            return (bits, bitDef['Width'], bitDef['Symbol'], bitDef['Values'][1], bitsValue)
                        else:
                            return (bits, bitDef['Width'],bitDef['Symbol'], bitDef['Values'][0], bitsValue)
                    else:
                        if len(bitDef['Values']) > 0:
                            return (bits, bitDef['Width'], bitDef['Symbol'], bitDef['Values'][bitsValue], bitsValue)  # enum
                        else:
                            return (bits, bitDef['Width'], bitDef['Symbol'], '', bitsValue)   # just use integer value
                except:
                    return (bits, 0, '', '', -1)